@@ -1,6 +1,6 @@
 <?php
 /*
-$HeadURL: http://svn.textpattern.com/development/4.0/textpattern/css.php $
+$HeadURL: http://svn.textpattern.com/releases/4.0.5/source/textpattern/css.php $
 $LastChangedRevision: 944 $
 */
 
