@@ -7,7 +7,7 @@
    Manual: http://scripts.incutio.com/xmlrpc/manual.php
    Made available under the Artistic License: http://www.opensource.org/licenses/artistic-license.php
 
-$HeadURL: http://svn.textpattern.com/development/4.0/textpattern/lib/IXRClass.php $
+$HeadURL: http://svn.textpattern.com/releases/4.0.5/source/textpattern/lib/IXRClass.php $
 $LastChangedRevision: 765 $
 */
 
