@@ -31,14 +31,9 @@
  *
  */
  
-/* changelog:
- * 2005-07-24	wet	initial release
- * 2005-07-27	wet	extracted file based thumbs as wet_thumb class
- */
-
 /*
-$HeadURL: http://svn.textpattern.com/development/4.0/textpattern/lib/class.thumb.php $
-$LastChangedRevision: 884 $
+$HeadURL: http://svn.textpattern.com/releases/4.0.5/source/textpattern/lib/class.thumb.php $
+$LastChangedRevision: 2329 $
 */
 
 $verbose = false;
@@ -128,7 +123,7 @@ class wet_thumb {
 	    }
 	    else {
 		$this->_DST['width']	= $this->longside;
-		$this->_DST['height']	= round($longside/($this->_SRC['width']/$this->_SRC['height']));
+		$this->_DST['height']	= round($this->longside/($this->_SRC['width']/$this->_SRC['height']));
 	    }
         }
 	elseif (is_numeric($this->shortside)) {
@@ -160,6 +155,11 @@ class wet_thumb {
 	$this->_DST['type'] = $this->_SRC['type']; 
 	$this->_DST['file'] = $outfile;
 
+	// Make sure we have enough memory if the image is large
+	if (max($this->_SRC['width'], $this->_SRC['height']) > 1024)
+		// this won't work on all servers but it's worth a try
+		ini_set('memory_limit', EXTRA_MEMORY);
+
 	// SRC einlesen
 	if ($this->_SRC['type'] == 1)	$this->_SRC['image'] = imagecreatefromgif($this->_SRC['file']);
 	elseif ($this->_SRC['type'] == 2)	$this->_SRC['image'] = imagecreatefromjpeg($this->_SRC['file']);
@@ -236,7 +236,13 @@ class wet_thumb {
 	
         if ($this->_DST['type'] == 1)	{
 	    imagetruecolortopalette($this->_DST['image'], false, 256);
-	    imagegif($this->_DST['image'], $this->_DST['file']);
+		if ( function_exists ('imagegif') ) {
+			imagegif($this->_DST['image'], $this->_DST['file']);
+		} else {
+			imagedestroy($this->_DST['image']);
+			imagedestroy($this->_SRC['image']);
+			return false;
+		}
 	}
 	elseif ($this->_DST['type'] == 2) {
 	    imagejpeg($this->_DST['image'], $this->_DST['file'], $this->quality);
@@ -290,9 +296,9 @@ class txp_thumb extends wet_thumb {
      * @param	$id	image id
      */
     function txp_thumb ($id) {
-        $rs = safe_row("*", "txp_image", "id='".doSlash($id)."' limit 1");
+        $id = assert_int($id);
+        $rs = safe_row('*', 'txp_image', 'id = '.$id.' limit 1');
         if ($rs) {
-	    // dmp ($rs);
             extract($rs);
             $this->m_ext = $ext;
             $this->m_id = $id;
@@ -308,23 +314,28 @@ class txp_thumb extends wet_thumb {
         if ( !isset($this->m_ext) ) return false;
         
         if ( parent::write ( IMPATH.$this->m_id.$this->m_ext, IMPATH.$this->m_id.'t'.$this->m_ext ) ) {
-	    safe_update("txp_image", "thumbnail='1'", "id='".doSlash($this->m_id)."'");
-	    return true;
-	}
+		    safe_update('txp_image', 'thumbnail = 1', 'id = '.$this->m_id);
+		    return true;
+		}
+		return false;
+    }
+
+     /**
+     * delete thumbnail
+     * @return	boolean, true indicates success
+     */
+    function delete( ) {
+        if (!isset($this->m_ext)) return false;
+        
+		if (unlink(IMPATH.$this->m_id.'t'.$this->m_ext)) {
+			safe_update('txp_image', 'thumbnail = 0', 'id = '.$this->m_id);
+	    	return true;
+		}
 	return false;
     }
-   
+  
 }
 
-
-/**
- * wet image manipulation tools
- */ 
-
-/* change log:
- *  2005-07-27  wet     initial creation
- */
-
 /**
  * Unsharp mask algorithm by Torstein Hnsi 2003 (thoensi_at_netcom_dot_no)
  * Christoph Erdmann: changed it a little, cause i could not reproduce the
@@ -402,22 +413,3 @@ function UnsharpMask($img, $amount, $radius, $threshold)    {
     }
     return $img;
 }
-
-
-
-
-
-/**
- * @todo
- * TxP tag wrappers
- * <txp:wet_thumb />
- * attributes (reflecting txp_thumb object properties):
- *  id: image id
- *  width: integer
- *  height: integer
- *  shortside: integer
- *  longside: integer
- *  link: "true"|"false"
- *  window: "true"|"false"
- *  hint: "true"|"false"
- */
