@@ -13,8 +13,8 @@
 
 	Use of this software denotes acceptance of the Textpattern license agreement 
 
-$HeadURL: http://svn.textpattern.com/development/4.0/textpattern/publish.php $
-$LastChangedRevision: 1186 $
+$HeadURL: http://svn.textpattern.com/releases/4.0.5/source/textpattern/publish.php $
+$LastChangedRevision: 2436 $
 
 */
 
@@ -40,9 +40,6 @@ $LastChangedRevision: 1186 $
 
 	ob_start();
 
-		// useful for clean urls with error-handlers
-	txp_status_header('200 OK');
-
     	// start the clock for runtime
 	$microstart = getmicrotime();
 
@@ -65,15 +62,18 @@ $LastChangedRevision: 1186 $
 	if (empty($path_to_site))
 		updateSitePath(dirname(dirname(__FILE__)));
 
+	if (!defined( 'PROTOCOL'))
+		define( 'PROTOCOL', ( ( serverSet('HTTPS') != '' ) ? 'https://' : 'http://') );
+		
 		// v1.0: this should be the definitive http address of the site	
 	if (!defined('hu'))
-		define("hu",'http://'.$siteurl.'/');
+		define("hu",PROTOCOL.$siteurl.'/');
 	
 		// v1.0 experimental relative url global
 	if (!defined('rhu'))
-		define("rhu",preg_replace("/http:\/\/.+(\/.*)\/?$/U","$1",hu));
+		define("rhu",preg_replace("|^https?://[^/]+|","",hu));
 
-		// 1.0: a new $here variable in the top-level index.php 
+		// 1.0: a new $here variable in the top-level index.php
 		// should let us know the server path to the live site
 		// let's save it to prefs
 	if (isset($here) and $path_to_site != $here) updateSitePath($here);
@@ -105,10 +105,10 @@ $LastChangedRevision: 1186 $
 	$pretext = !isset($pretext) ? array() : $pretext;
 	$pretext = array_merge($pretext, pretext($s,$prefs));
 	extract($pretext);
-	
+
 	// Now that everything is initialized, we can crank down error reporting
 	set_error_level($production_status);
-	
+
 	if (gps('parentid') && gps('submit')) {
 		saveComment();
 	} elseif (gps('parentid') and $comments_mode==1) { // popup comments?
@@ -118,6 +118,7 @@ $LastChangedRevision: 1186 $
 
 	// we are dealing with a download
 	if (@$s == 'file_download') {
+		callback_event('file_download');
 		if (!isset($file_error)) {
 
 				$fullpath = build_file_path($file_base_path,$filename);
@@ -129,8 +130,9 @@ $LastChangedRevision: 1186 $
 					$filesize = filesize($fullpath); $sent = 0;
 					header('Content-Description: File Download');
 					header('Content-Type: application/octet-stream');
-					header('Content-Length: ' . $filesize);
-					header('Content-Disposition: attachment; filename="' . basename($filename) . '"');
+					header('Content-Disposition: attachment; filename="' . basename($filename) . '"; size = "'.$filesize.'"');
+					// Fix for lame IE 6 pdf bug on servers configured to send cache headers
+					header('Cache-Control: private');
 					@ini_set("zlib.output_compression", "Off");
 					@set_time_limit(0);
 					@ignore_user_abort(true);
@@ -143,12 +145,15 @@ $LastChangedRevision: 1186 $
 						fclose($file);
 						// record download
 						if ((connection_status()==0) and !connection_aborted() ) {
-							safe_update("txp_file", "downloads=downloads+1", "id='".intval($id)."'");
+							safe_update("txp_file", "downloads=downloads+1", 'id='.intval($id));
+							log_hit('200');
 						} else {
-							$pretext['request_uri'] .= "#aborted-at-".floor($sent*100/$filesize)."%";
-							logit();
+							$pretext['request_uri'] .= ($sent >= $filesize)
+								? '#aborted'
+								: "#aborted-at-".floor($sent*100/$filesize)."%";
+							log_hit('200');
 						}
-					}      				
+					}
 				} else {
 					$file_error = 404;
 				}
@@ -168,34 +173,21 @@ $LastChangedRevision: 1186 $
 				break;
 			}
 		}
-		
+
 		// download done
 		exit(0);
 	}
-	
 
-	if(!isset($nolog) && $status != '404') {
-		if($logging == 'refer') { 
-			logit('refer'); 
-		} elseif ($logging == 'all') {
-			logit();
-		}
-	}
 
-/* FIXME: have to convert mysql-timestamps to unixtimestamps first. Are timezones ok?
-	if($send_lastmod) {
-		$last = gmdate("D, d M Y H:i:s \G\M\T",$lastmod);
-		header("Last-Modified: $last");
+	// send 304 Not Modified if appropriate
+	handle_lastmod();
+
+	// log the page view
+	log_hit($status);
+
 
-		$hims = serverset('HTTP_IF_MODIFIED_SINCE');
-		if ($hims == $last) {
-			header("HTTP/1.1 304 Not Modified");
-			exit; 
-		}
-	}
-*/
 // -------------------------------------------------------------
-	function preText($s,$prefs) 
+	function preText($s,$prefs)
 	{
 		extract($prefs);
 
@@ -214,30 +206,38 @@ $LastChangedRevision: 1186 $
 		$out =  makeOut('id','s','c','q','pg','p','month','author');
 
 			// some useful vars for taghandlers, plugins
-		$out['request_uri'] = serverSet('REQUEST_URI');
+		$out['request_uri'] = preg_replace("|^https?://[^/]+|i","",serverSet('REQUEST_URI'));
 		$out['qs'] = serverSet('QUERY_STRING');
-			// IIS - can someone confirm whether or not this works?
-		if (!$out['request_uri'] and $argv = serverSet('argv'))
-			$out['request_uri'] = @substr($argv[0], strpos($argv[0], ';' + 1));
-			// IIS again - given that the above doesn't always seem to help...
+			// IIS fix 
 		if (!$out['request_uri'] and serverSet('SCRIPT_NAME'))
 			$out['request_uri'] = serverSet('SCRIPT_NAME').( (serverSet('QUERY_STRING')) ? '?'.serverSet('QUERY_STRING') : '');
+			// another IIS fix
+		if (!$out['request_uri'] and serverSet('argv'))
+		{
+			$argv = serverSet('argv');
+			$out['request_uri'] = @substr($argv[0], strpos($argv[0], ';') + 1);
+		}
 
 			// define the useable url, minus any subdirectories.
 			// this is pretty fugly, if anyone wants to have a go at it - dean
-		$out['subpath'] = $subpath = preg_quote(preg_replace("/http:\/\/.*(\/.*)/Ui","$1",hu),"/");
-		$out['req'] = $req = preg_replace("/^$subpath/i","/",serverSet('REQUEST_URI'));
+		$out['subpath'] = $subpath = preg_quote(preg_replace("/https?:\/\/.*(\/.*)/Ui","$1",hu),"/");
+		$out['req'] = $req = preg_replace("/^$subpath/i","/",$out['request_uri']);
 
 		$is_404 = 0;
 
 			// if messy vars exist, bypass url parsing
-		if (!$out['id'] && !$out['s']) {
+		if (!$out['id'] && !$out['s'] && !(txpinterface=='css') &&! ( txpinterface=='admin') ) {
+			
+			// return clean URL test results for diagnostics
+			if (gps('txpcleantest')) {
+				exit(show_clean_test($out));
+			}
 
 			extract(chopUrl($req));
 	
 				//first we sniff out some of the preset url schemes
-			if (!empty($u1)) {
-	
+			if (strlen($u1)) {
+
 				switch($u1) {
 	
 					case 'atom':
@@ -249,10 +249,10 @@ $LastChangedRevision: 1186 $
 					// urldecode(strtolower(urlencode())) looks ugly but is the only way to
 					// make it multibyte-safe without breaking backwards-compatibility
 					case urldecode(strtolower(urlencode(gTxt('section')))):
-						$out['s'] = (ckEx('section',$u2)) ? $u2 : ''; break;
+						$out['s'] = (ckEx('section',$u2)) ? $u2 : ''; $is_404 = empty($out['s']); break;
 	
 					case urldecode(strtolower(urlencode(gTxt('category')))):
-						$out['c'] = (ckEx('category',$u2)) ? $u2 : ''; break;
+						$out['c'] = (ckEx('category',$u2)) ? $u2 : ''; $is_404 = empty($out['c']); break;
 	
 					case urldecode(strtolower(urlencode(gTxt('author')))):
 						$out['author'] = (!empty($u2)) ? $u2 : ''; break;
@@ -262,9 +262,6 @@ $LastChangedRevision: 1186 $
 						$out['s'] = 'file_download';
 						$out['id'] = (!empty($u2)) ? $u2 : ''; break;
 					
-					case 'p':
-						$out['p'] = (is_numeric($u2)) ? $u2 : ''; break;
-					
 					default:
 						// then see if the prefs-defined permlink scheme is usable
 						switch ($permlink_mode) {
@@ -275,7 +272,7 @@ $LastChangedRevision: 1186 $
 									$is_404 = empty($out['s']);
 								}
 								else {
-									$rs = ckExID($u2);
+									$rs = lookupByIDSection($u2, $u1);
 									$out['s'] = @$rs['Section'];
 									$out['id'] = @$rs['ID'];
 									$is_404 = (empty($out['s']) or empty($out['id']));
@@ -349,23 +346,45 @@ $LastChangedRevision: 1186 $
 		}
 		else {
 			// Messy mode, but prevent to get the id for file_downloads
-			if ($out['id'] && !$out['s'])
-				$out['s'] = safe_field('section', 'textpattern', "ID='".doSlash($out['id'])."'");
+			if ($out['id'] && !$out['s']) {
+				$rs = lookupByID($out['id']);
+				$out['id'] = (!empty($rs['ID'])) ? $rs['ID'] : '';
+				$out['s'] = (!empty($rs['Section'])) ? $rs['Section'] : '';
+				$is_404 = (empty($out['s']) or empty($out['id']));
+			}
 		}
 
 		// Resolve AuthorID from Authorname
 		if ($out['author'])
-			$out['author'] = safe_field('name','txp_users',"RealName like '".doSlash($out['author'])."'");
+		{
+			$name = urldecode(strtolower(urlencode($out['author'])));
+		
+			$name = safe_field('name', 'txp_users', "RealName like '".doSlash($out['author'])."'");
+
+			if ($name)
+			{
+				$out['author'] = $name;
+			}
+
+			else
+			{
+				$out['author'] = '';
+				$is_404 = true;
+			}
+		}
 
 		// Stats: found or not
 		$out['status'] = ($is_404 ? '404' : '200');
+
+		$out['pg'] = is_numeric($out['pg']) ? intval($out['pg']) : '';
+		$out['id'] = is_numeric($out['id']) ? intval($out['id']) : '';
 		
 		if ($out['s'] == 'file_download') {
 			// get id of potential filename
 			if (!is_numeric($out['id'])) {
 				$rs = safe_row("*", "txp_file", "filename='".doSlash($out['id'])."'");
 			} else {
-				$rs = safe_row("*", "txp_file", "id='".intval($out['id'])."'");
+				$rs = safe_row("*", "txp_file", 'id='.intval($out['id']));
 			}
 
 			$out = ($rs)? array_merge($out, $rs) : array('s'=>'file_download','file_error'=> 404);
@@ -384,26 +403,26 @@ $LastChangedRevision: 1186 $
 			// by this point we should know the section, so grab its page and css
 		$rs = safe_row("*", "txp_section", "name = '".doSlash($s)."' limit 1");
 		$out['page'] = @$rs['page'];		
-// Remove the below line if nothing breaks
-// 		$out['css']  = @$rs['css'];
 
 		if(is_numeric($id)) {
-			$idrs = safe_row("Posted, AuthorID, Keywords","textpattern","ID=".doSlash($id));
-			extract($idrs);
+			$a = safe_row('*, unix_timestamp(Posted) as uPosted', 'textpattern', 'ID='.intval($id).' and Status = 4');
+			if ($a) {
+				$Posted             = @$a['Posted'];
+				$out['id_keywords'] = @$a['Keywords'];
+				$out['id_author']   = @$a['AuthorID'];
+				populateArticleData($a);
 
-			if ($np = getNextPrev($id, $Posted, $s))
-				$out = array_merge($out, $np);
-
-			$out['id_keywords'] = $Keywords; 
-			$out['id_author']   = get_author_name($AuthorID); 
+				if ($np = getNextPrev($id, $Posted, $s))
+					$out = array_merge($out, $np);
+			}
 		}
 
-		$out['path_from_root'] = $path_from_root; // these are deprecated as of 1.0
-		$out['pfr']            = $path_from_root; // leaving them here for plugin compat
+		$out['path_from_root'] = rhu; // these are deprecated as of 1.0
+		$out['pfr']            = rhu; // leaving them here for plugin compat
 
 		$out['path_to_site']   = $path_to_site;
 		$out['permlink_mode']  = $permlink_mode;
-		$out['sitename']       = htmlspecialchars($sitename);
+		$out['sitename']       = $sitename;
 
 		return $out; 
 
@@ -415,7 +434,9 @@ $LastChangedRevision: 1186 $
 // -------------------------------------------------------------
 	function textpattern() 
 	{
-		global $pretext,$microstart,$prefs,$qcount,$qtime,$production_status,$txptrace,$siteurl;
+		global $pretext,$microstart,$prefs,$qcount,$qtime,$production_status,$txptrace,$siteurl,$has_article_tag;
+
+		$has_article_tag = false;
 
 		callback_event('textpattern');
 
@@ -426,16 +447,23 @@ $LastChangedRevision: 1186 $
 		if (!$html) 
 			txp_die(gTxt('unknown_section'), '404');
 
+		// useful for clean urls with error-handlers
+		txp_status_header('200 OK');
+
 		trace_add('['.gTxt('page').': '.$pretext['page'].']');
 		set_error_handler("tagErrorHandler");
 		$pretext['secondpass'] = false;
 		$html = parse($html);
 		$pretext['secondpass'] = true;
-		trace_add('[ --- '.gTxt('secondpass').' --- ]');
+		trace_add('[ ~~~ '.gTxt('secondpass').' ~~~ ]');
 		$html = parse($html); // the function so nice, he ran it twice
-		restore_error_handler();
 		$html = ($prefs['allow_page_php_scripting']) ? evalString($html) : $html;
 
+		// make sure the page has an article tag if necessary
+		if (!$has_article_tag and $production_status != 'live' and (!empty($pretext['id']) or !empty($pretext['c']) or !empty($pretext['q']) or !empty($pretext['pg'])))
+			trigger_error(gTxt('missing_article_tag', array('{page}' => $pretext['page'])));
+		restore_error_handler();
+
 		header("Content-type: text/html; charset=utf-8");
 		echo $html;
 
@@ -448,6 +476,8 @@ $LastChangedRevision: 1186 $
 			if (!empty($txptrace) and is_array($txptrace))
 				echo n, comment('txp tag trace: '.n.join(n, $txptrace).n);
 		}
+
+		callback_event('textpattern_end');
 	}
 
 // -------------------------------------------------------------
@@ -469,7 +499,13 @@ $LastChangedRevision: 1186 $
 
 // -------------------------------------------------------------
 	function article($atts)
-	{		
+	{
+		global $is_article_body, $has_article_tag;
+		if ($is_article_body) {
+			trigger_error(gTxt('article_tag_illegal_body'));
+			return '';
+		}
+		$has_article_tag = true;
 		return parseArticles($atts);
 	}
 
@@ -479,17 +515,23 @@ $LastChangedRevision: 1186 $
 		global $pretext, $prefs, $txpcfg;
 		extract($pretext);
 		extract($prefs);
+		$customFields = getCustomFields();
+		$customlAtts = array_null(array_flip($customFields));
+
 		//getting attributes
 		$theAtts = lAtts(array(
 			'form'      => 'default',
+			'listform'  => '',
+			'searchform'=> '',
 			'limit'     => 10,
 			'pageby'    => '',
 			'category'  => '',
 			'section'   => '',
 			'excerpted' => '',
 			'author'    => '',
+			'sort'      => '',
 			'sortby'    => '',
-			'sortdir'   => 'desc',
+			'sortdir'   => '',
 			'month'     => '',
 			'keywords'  => '',
 			'frontpage' => '',
@@ -501,8 +543,8 @@ $LastChangedRevision: 1186 $
 			'searchsticky' => 0,
 			'allowoverride' => (!$q and !$iscustom),
 			'offset'    => 0,
-		),$atts);		
-		
+		)+$customlAtts,$atts);
+
 		// if an article ID is specified, treat it as a custom list
 		$iscustom = (!empty($theAtts['id'])) ? true : $iscustom;
 
@@ -518,7 +560,7 @@ $LastChangedRevision: 1186 $
 			$theAtts['excerpted'] = '';			
 		}
 		extract($theAtts);
-		
+
 		$pageby = (empty($pageby) ? $limit : $pageby);
 
 		// treat sticky articles differently wrt search filtering, etc
@@ -536,19 +578,36 @@ $LastChangedRevision: 1186 $
 
 			// searchall=0 can be used to show search results for the current section only
 			if ($searchall) $section = '';
-			if (!$sortby) $sortby='score';
+			if (!$sort) $sort='score desc';
 		}
 		else {
 			$match = $search = '';
-			if (!$sortby) $sortby='Posted';
+			if (!$sort) $sort='Posted desc';
+		}
+
+		// for backwards compatibility
+		// sortby and sortdir are deprecated
+		if ($sortby)
+		{
+			if (!$sortdir)
+			{
+				$sortdir = 'desc';
+			}
+
+			$sort = "$sortby $sortdir";
+		}
+
+		elseif ($sortdir)
+		{
+			$sort = "Posted $sortdir";
 		}
 
 		//Building query parts
 		$frontpage = ($frontpage and !$q) ? filterFrontPage() : '';		
-		$category  = (!$category)  ? '' : " and ((Category1='".doslash($category)."') or (Category2='".doSlash($category)."')) ";
-		$section   = (!$section)   ? '' : " and Section = '".doslash($section)."'";
+		$category  = (!$category)  ? '' : " and ((Category1='".doSlash($category)."') or (Category2='".doSlash($category)."')) ";
+		$section   = (!$section)   ? '' : " and Section = '".doSlash($section)."'";
 		$excerpted = ($excerpted=='y')  ? " and Excerpt !=''" : '';
-		$author    = (!$author)    ? '' : " and AuthorID = '".doslash($author)."'";
+		$author    = (!$author)    ? '' : " and AuthorID = '".doSlash($author)."'";
 		$month     = (!$month)     ? '' : " and Posted like '".doSlash($month)."%'";
 		$id        = (!$id)        ? '' : " and ID = '".intval($id)."'";
 		switch ($time) {
@@ -557,41 +616,36 @@ $LastChangedRevision: 1186 $
 			case 'future':
 				$time = " and Posted > now()"; break;
 			default:
-				$time = " and Posted < now()";
+				$time = " and Posted <= now()";
 		}
-		if (!is_numeric($status))
-			$status = getStatusNum($status);
 			
 		$custom = '';
 
-		// trying custom fields here
-		$customFields = getCustomFields();
-		
 		if ($customFields) {
 			foreach($customFields as $cField) {
 				if (isset($atts[$cField]))
 					$customPairs[$cField] = $atts[$cField];
 			}
 			if(!empty($customPairs)) {
-				$custom =  buildCustomSql($customFields,$customPairs);
-			} else $custom = '';
+				$custom = buildCustomSql($customFields,$customPairs);
+			}
 		}
 
 		//Allow keywords for no-custom articles. That tagging mode, you know
 		if ($keywords) {
-			$keys = split(',',$keywords);
+			$keys = doSlash(array_map('trim', split(',' ,$keywords)));
 			foreach ($keys as $key) {
-				$keyparts[] = " Keywords like '%".doSlash(trim($key))."%'";
+				$keyparts[] = "FIND_IN_SET('".$key."',Keywords)";
 			}
 			$keywords = " and (" . join(' or ',$keyparts) . ")"; 
 		}
 
 		if ($q and $searchsticky)
-			$statusq = " and Status >= '4'";
+			$statusq = ' and Status >= 4';
 		elseif ($id)
-			$statusq = " and Status >= '4'";
+			$statusq = ' and Status >= 4';
 		else
-			$statusq = " and Status='".doSlash($status)."'";
+			$statusq = ' and Status = '.intval($status);
 
 		$where = "1" . $statusq. $time.
 			$search . $id . $category . $section . $excerpted . $month . $author . $keywords . $custom . $frontpage;
@@ -599,36 +653,35 @@ $LastChangedRevision: 1186 $
 		//do not paginate if we are on a custom list
 		if (!$iscustom and !$issticky)
 		{
-			$total = safe_count('textpattern',$where) - $offset;
+			$grand_total = safe_count('textpattern',$where);
+			$total = $grand_total - $offset;
 			$numPages = ceil($total/$pageby);  
 			$pg = (!$pg) ? 1 : $pg;
-			$pgoffset = $offset + (($pg - 1) * $pageby).', ';	
+			$pgoffset = $offset + (($pg - 1) * $pageby);	
 			// send paging info to txp:newer and txp:older
 			$pageout['pg']       = $pg;
 			$pageout['numPages'] = $numPages;
 			$pageout['s']        = $s;
 			$pageout['c']        = $c;
+			$pageout['grand_total'] = $grand_total;
 			$pageout['total']    = $total;
 
-			global $thispage;	
-			$thispage = $pageout;
+			global $thispage;
+			if (empty($thispage))
+				$thispage = $pageout;
 			if ($pgonly)
 				return;
 		}else{
-			$pgoffset = $offset . ', ';
+			$pgoffset = $offset;
 		}
 
 		$rs = safe_rows_start("*, unix_timestamp(Posted) as uPosted".$match, 'textpattern', 
-		$where. ' order by ' . doslash($sortby) . ' ' . doSlash($sortdir) . ' limit ' . doSlash($pgoffset.$limit));
-		// alternative form override for search or list
+		$where.' order by '.doSlash($sort).' limit '.intval($pgoffset).', '.intval($limit));
+		// get the form name
 		if ($q and !$iscustom and !$issticky)
-			$form = gAtt($atts, 'searchform', 'search_results');
+			$fname = ($searchform ? $searchform : 'search_results');
 		else
-			$form = gAtt($atts, 'listform', $form);         
-		// might be a form preview, otherwise grab it from the db
-		$form = (isset($_POST['Form']))
-		?	gps('Form')
-		:	fetch_form($form);
+			$fname = ($listform ? $listform : $form);
 
 		if ($rs) {
 			$count = 0;
@@ -640,85 +693,96 @@ $LastChangedRevision: 1186 $
 				global $thisarticle, $uPosted, $limit;
 				$thisarticle['is_first'] = ($count == 1);
 				$thisarticle['is_last'] = ($count == numRows($rs));
-				// define the article form
-				$article = ($allowoverride and $a['override_form']) 
-				?	fetch_form($a['override_form'])
-				:	$form;
 
-				$articles[] = parse($article);
-				
+				if (@constant('txpinterface') === 'admin' and gps('Form')) {
+					$articles[] = parse(gps('Form'));
+				}
+				elseif ($allowoverride and $a['override_form']) {
+					$articles[] = parse_form($a['override_form']);
+				}
+				else {
+					$articles[] = parse_form($fname);
+				}
+
 				// sending these to paging_link(); Required?
 				$uPosted = $a['uPosted'];
-				$limit = $limit;
 
 				unset($GLOBALS['thisarticle']);
-				unset($GLOBALS['theseatts']);//Required?				
 			}
-			
+
 			return join('',$articles);
 		}
 	}
 
 // -------------------------------------------------------------
-	function filterFrontPage() 
+
+	function filterFrontPage()
 	{
-        static $filterFrontPage;
+		static $filterFrontPage;
 
-        if (isset($filterFrontPage)) return $filterFrontPage;
-
-		$rs = safe_column("name","txp_section", "on_frontpage != '1'");
-		if ($rs) {
-			foreach($rs as $name) $filters[] = "and Section != '".doSlash($name)."'";	
-			$filterFrontPage = join(' ',$filters);
-            return $filterFrontPage;
+		if (isset($filterFrontPage)) {
+			return $filterFrontPage;
 		}
-        $filterFrontPage = false;
+
+		$filterFrontPage = false;
+
+		$rs = safe_column('name', 'txp_section', "on_frontpage != '1'");
+
+		if ($rs) {
+			$filters = array();
+
+			foreach ($rs as $name) {
+				$filters[] = " and Section != '".doSlash($name)."'";
+			}
+			
+			$filterFrontPage = join('', $filters);
+		}
+
 		return $filterFrontPage;
 	}
 
-
 // -------------------------------------------------------------
 	function doArticle($atts) 
 	{
-		global $pretext,$prefs;
+		global $pretext,$prefs, $thisarticle;
 		extract($prefs);
 		extract($pretext);
 
-		$preview = ps('preview');
-		$parentid = ps('parentid');
+		extract(gpsa(array('parentid', 'preview')));
 
 		extract(lAtts(array(
-			'form' => 'default',
-			'status' => '',
-		),$atts));		
+			'allowoverride' => '1',
+			'form'          => 'default',
+			'status'        => '4',
+		),$atts, 0));		
 
-		if ($status and !is_numeric($status))
-			$status = getStatusNum($status);
+		if ($status or empty($thisarticle) or $thisarticle['thisid'] != $id) {
+			$thisarticle = NULL;
+			if ($status and !is_numeric($status))
+				$status = getStatusNum($status);
 
-		$q_status = ($status ? "and Status='".doSlash($status)."'" : 'and Status in (4,5)');
+			$q_status = ($status ? 'and Status = '.intval($status) : 'and Status in (4,5)');
 
-		$rs = safe_row("*, unix_timestamp(Posted) as uPosted", 
-				"textpattern", "ID='".intval($id)."' $q_status limit 1");
+			$rs = safe_row("*, unix_timestamp(Posted) as uPosted", 
+					"textpattern", 'ID = '.intval($id)." $q_status limit 1");
 
-		if ($rs) {
-			extract($rs);
-			populateArticleData($rs);
-			global $thisarticle;
+			if ($rs) {
+				extract($rs);
+				populateArticleData($rs);
+			}
+		}
+
+		if (!empty($thisarticle)) {
+			extract($thisarticle);
 			$thisarticle['is_first'] = 1;
 			$thisarticle['is_last'] = 1;
 
-			// define the article form
-			$article = fetch_form(($override_form) ? $override_form : $form);
+			$form = ($allowoverride and $override_form) ? $override_form : $form;
 
-			if ($preview && $parentid) {
-				$article = '<txp:comments_preview bc="1" id="'.$parentid.'" />'.$article;
-			}
-
-			$article = parse($article);
+			$article = parse_form($form);
 
 			if ($use_comments and $comments_auto_append) {
-				$f = fetch_form('comments_display');
-				$article .= parse($f);
+				$article .= parse_form('comments_display');
 			}
 
 			
@@ -740,7 +804,9 @@ $LastChangedRevision: 1186 $
 		global $pretext, $is_article_list;
 		$old_ial = $is_article_list;
 		$is_article_list = ($pretext['id'] && !$iscustom)? false : true;
+		article_push();
 		$r = ($is_article_list)? doArticles($atts, $iscustom) : doArticle($atts);
+		article_pop();
 		$is_article_list = $old_ial;
 
 		return $r;
@@ -768,30 +834,28 @@ $LastChangedRevision: 1186 $
 		$out['keywords']        = $Keywords;
 		$out['article_image']   = $Image;
 		$out['comments_count']  = $comments_count;
-
+		$out['body']            = $Body_html;
+		$out['excerpt']         = $Excerpt_html;
+		$out['override_form']		= $override_form;
 
 		$custom = getCustomFields();
 		if ($custom) {
 			foreach ($custom as $i => $name)
 				$out[$name] = $rs['custom_' . $i];
 		}
-			
-		global $thisarticle, $is_article_body;
-		$thisarticle = $out;
-		$is_article_body = 1;		
-		$thisarticle['body'] = parse($Body_html);
-		$thisarticle['excerpt'] = parse($Excerpt_html);
-		$is_article_body = 0;
-
+		
+		global $thisarticle;
+		$thisarticle = $out;	
 	}
 
 // -------------------------------------------------------------
 	function getNeighbour($Posted, $s, $type) 
 	{
 		$type = ($type == '>') ? '>' : '<';
+		$safe_name = safe_pfx('textpattern');
 		$q = array(
 			"select ID, Title, url_title, unix_timestamp(Posted) as uposted
-			from ".PFX."textpattern where Posted $type '".doSlash($Posted)."'",
+			from ".$safe_name." where Posted $type '".doSlash($Posted)."'",
 			($s!='' && $s!='default') ? "and Section = '".doSlash($s)."'" : filterFrontPage(),
 			'and Status=4 and Posted < now() order by Posted',
 			($type=='<') ? 'desc' : 'asc',
@@ -812,7 +876,7 @@ $LastChangedRevision: 1186 $
 		if (!$id) {
 			$current = safe_row('ID, Posted', 'textpattern', 
 				'1=1 '.
-				(($s!='' && $s!='default') ? "Section = '".doSlash($s)."'" : filterFrontPage()).
+				(($s!='' && $s!='default') ? "and Section = '".doSlash($s)."'" : filterFrontPage()).
 				'and Status=4 and Posted < now() order by Posted desc limit 1');
 			if ($current) {
 				$id = $current['ID'];
@@ -844,27 +908,6 @@ $LastChangedRevision: 1186 $
 		return $out;
 	}
 
-// -------------------------------------------------------------
-	function since($stamp) 
-	{
-		$diff = (time() - $stamp);
-		if ($diff <= 3600) {
-			$mins = round($diff / 60);
-			$since = ($mins <= 1) 
-			?	($mins==1)
-				?	'1 '.gTxt('minute')
-				:	gTxt('a_few_seconds')
-			:	"$mins ".gTxt('minutes');
-		} else if (($diff <= 86400) && ($diff > 3600)) {
-			$hours = round($diff / 3600);
-			$since = ($hours <= 1) ? '1 '.gTxt('hour') : "$hours ".gTxt('hours');
-		} else if ($diff >= 86400) {
-			$days = round($diff / 86400);
-			$since = ($days <= 1) ? "1 ".gTxt('day') : "$days ".gTxt('days');
-		}
-		return $since.' '.gTxt('ago'); // sorry, this needs to be hacked until a truly multilingual version is done
-	}
-
 // -------------------------------------------------------------
 	function lastMod() 
 	{
@@ -880,42 +923,65 @@ $LastChangedRevision: 1186 $
 	}
 
 // -------------------------------------------------------------
+
 	function processTags($matches)
 	{
 		global $pretext, $production_status, $txptrace, $txptracelevel, $txp_current_tag;
 
 		$tag = $matches[1];
 
-		$atts = (isset($matches[2])) ? splat($matches[2]) : '';
-		$thing = (isset($matches[4])) ? $matches[4] : '';
+		$trouble_makers = array(
+			'link'
+		);
+
+		if (in_array($tag, $trouble_makers))
+		{
+			$tag = 'tpt_'.$tag;
+		}
+
+		$atts = isset($matches[2]) ? splat($matches[2]) : '';
+		$thing = isset($matches[4]) ? $matches[4] : null;
 
 		$old_tag = @$txp_current_tag;
+
 		$txp_current_tag = '<txp:'.$tag.
 			($atts ? $matches[2] : '').
 			($thing ? '>' : '/>');
 
 		trace_add($txp_current_tag);
 		@++$txptracelevel;
+
 		if ($production_status == 'debug')
+		{
 			maxMemUsage(trim($matches[0]));
+		}
 
 		$out = '';
 
-		if ($thing) {
-			if (function_exists($tag)) $out = $tag($atts,$thing,$matches[0]);
-			elseif (isset($pretext[$tag])) $out = $pretext[$tag];
-			else trigger_error(gTxt('unknown_tag'), E_USER_WARNING);
-		} else {
-			if (function_exists($tag)) $out = $tag($atts,null,$matches[0]);
-			elseif (isset($pretext[$tag])) $out = $pretext[$tag];
-			else trigger_error(gTxt('unknown_tag'), E_USER_WARNING);
+		if (function_exists($tag))
+		{
+			$out = $tag($atts, $thing, $matches[0]);
+		}
+
+		elseif (isset($pretext[$tag]))
+		{
+			$out = escape_output($pretext[$tag]);
+		}
+
+		else
+		{
+			trigger_error(gTxt('unknown_tag', array('{tag}' => $tag)), E_USER_WARNING);
 		}
 
 		@--$txptracelevel;
+
 		if (isset($matches[4]))
+		{
 			trace_add('</txp:'.$tag.'>');
+		}
 
 		$txp_current_tag = $old_tag;
+
 		return $out;
 	}
 
@@ -928,26 +994,31 @@ $LastChangedRevision: 1186 $
 	}
 
 // -------------------------------------------------------------
-	function evalString($html) 
+	function evalString($html)
 	{
+		global $prefs;
 		if (strpos($html, chr(60).'?php') !== false) {
-			$html = eval(' ?'.chr(62).$html.chr(60).'?php ');
+			trigger_error(gTxt('raw_php_deprecated'), E_USER_WARNING);
+			if (!empty($prefs['allow_raw_php_scripting']))
+				$html = eval(' ?'.chr(62).$html.chr(60).'?php ');
+			else
+				trigger_error(gTxt('raw_php_disabled'), E_USER_WARNING);
 		}
-		return $html;	
+		return $html;
 	}
 
 // -------------------------------------------------------------
 	function getCustomFields()
 	{
 		global $prefs;
+		$out = array();
 		$i = 0;
-		while ($i < 10) {
-			$i++;
+		for ($i=1; $i<=10; $i++) {
 			if (!empty($prefs['custom_'.$i.'_set'])) {
-				$out[$i] = $prefs['custom_'.$i.'_set'];
+				$out[$i] = strtolower($prefs['custom_'.$i.'_set']);
 			}
 		}
-		return (!empty($out)) ? $out : false;
+		return $out;
 	}
 	
 // -------------------------------------------------------------
@@ -967,7 +1038,7 @@ $LastChangedRevision: 1186 $
 	}
 
 // -------------------------------------------------------------
-	function getStatusNum($name) 
+	function getStatusNum($name)
 	{
 		$labels = array('draft' => 1, 'hidden' => 2, 'pending' => 3, 'live' => 4, 'sticky' => 5);
 		$status = strtolower($name);
@@ -984,38 +1055,46 @@ $LastChangedRevision: 1186 $
 // -------------------------------------------------------------
 	function ckExID($val,$debug='') 
 	{
-		return safe_row("ID,Section",'textpattern',"ID = ".doSlash($val)." limit 1",$debug);
+		return safe_row("ID,Section",'textpattern','ID = '.intval($val).' and Status >= 4 limit 1',$debug);
 	}
 
 // -------------------------------------------------------------
 	function lookupByTitle($val,$debug='') 
 	{
-		return safe_row("ID,Section",'textpattern',"url_title like '".doSlash($val)."' limit 1",$debug);
+		return safe_row("ID,Section",'textpattern',"url_title like '".doSlash($val)."' and Status >= 4 limit 1",$debug);
 	}
 // -------------------------------------------------------------
 	function lookupByTitleSection($val,$section,$debug='') 
 	{
-		return safe_row("ID,Section",'textpattern',"url_title like '".doSlash($val)."' AND Section='$section' limit 1",$debug);
+		return safe_row("ID,Section",'textpattern',"url_title like '".doSlash($val)."' AND Section='".doSlash($section)."' and Status >= 4 limit 1",$debug);
+	}	
+
+// -------------------------------------------------------------
+
+	function lookupByIDSection($id, $section, $debug = '') 
+	{
+		return safe_row('ID, Section', 'textpattern', 
+			'ID = '.intval($id)." and Section = '".doSlash($section)."' and Status >= 4 limit 1", $debug);
 	}	
 
 // -------------------------------------------------------------
 	function lookupByID($id,$debug='') 
 	{
-		return safe_row("ID,Section",'textpattern',"ID = '".doSlash($id)."' limit 1",$debug);
+		return safe_row("ID,Section",'textpattern','ID = '.intval($id).' and Status >= 4 limit 1',$debug);
 	}
 
 // -------------------------------------------------------------
 	function lookupByDateTitle($when,$title,$debug='') 
 	{
 		return safe_row("ID,Section","textpattern",
-		"posted like '".doSlash($when)."%' and url_title like '".doSlash($title)."' limit 1");
+		"posted like '".doSlash($when)."%' and url_title like '".doSlash($title)."' and Status >= 4 limit 1");
 	}
 
 // -------------------------------------------------------------
 	function makeOut() 
 	{
 		foreach(func_get_args() as $a) {
-			$array[$a] = htmlspecialchars(gps($a));
+			$array[$a] = strval(gps($a));
 		}
 		return $array;
 	}
@@ -1023,17 +1102,17 @@ $LastChangedRevision: 1186 $
 // -------------------------------------------------------------
 	function chopUrl($req) 
 	{
-		$req = urldecode(strtolower($req));
+		$req = strtolower($req);
 		//strip off query_string, if present
 		$qs = strpos($req,'?');
 		if ($qs) $req = substr($req, 0, $qs);
 		$req = preg_replace('/index\.php$/', '', $req);
-		$r = explode('/',$req);
-		$o['u0'] = (!empty($r[0])) ? $r[0] : '';
-		$o['u1'] = (!empty($r[1])) ? $r[1] : '';
-		$o['u2'] = (!empty($r[2])) ? $r[2] : '';
-		$o['u3'] = (!empty($r[3])) ? $r[3] : '';
-		$o['u4'] = (!empty($r[4])) ? $r[4] : '';
+		$r = array_map('urldecode', explode('/',$req));
+		$o['u0'] = (isset($r[0])) ? $r[0] : '';
+		$o['u1'] = (isset($r[1])) ? $r[1] : '';
+		$o['u2'] = (isset($r[2])) ? $r[2] : '';
+		$o['u3'] = (isset($r[3])) ? $r[3] : '';
+		$o['u4'] = (isset($r[4])) ? $r[4] : '';
 
 		return $o;
 	}
