@@ -227,7 +227,7 @@ eod;
 	
 		while ($a = mysql_fetch_array($rs)){
 			extract($a);
-			$url_title = addslashes(stripSpace($Title));
+			$url_title = addslashes(stripSpace($Title,1));
 			safe_update("textpattern","url_title = '$url_title'","ID=$ID");
 		}
 
@@ -393,7 +393,7 @@ eod;
 		$prefs['blog_uid'] = md5(uniqid(rand(),true));
 		safe_insert('txp_prefs',"name='blog_uid', val='".$prefs['blog_uid']."', prefs_id='1'");
 	}
-	if (!safe_field('val','txp_prefs',"name='blog_mail_uid'"))
+	if (!safe_field('name','txp_prefs',"name='blog_mail_uid'"))
 	{
 		$mail = safe_field('email', 'txp_users', "privs='1' LIMIT 1");
 		safe_insert('txp_prefs',"name='blog_mail_uid', val='$mail', prefs_id='1'");
@@ -638,7 +638,7 @@ EOF;
 	safe_update('txp_prefs',"type = '1'","name = 'file_base_path'");
 
 	// 1.0: add option to override charset for emails (ISO-8559-1 instead of UTF-8)
-	if (!safe_field('val','txp_prefs',"name='override_emailcharset'"))
+	if (!safe_field('name','txp_prefs',"name='override_emailcharset'"))
 	{		
 		safe_insert('txp_prefs',"name='override_emailcharset', val='0', prefs_id='1', type='1', event='admin', position='".doSlash($maxpos)."', html='yesnoradio'");
 	}
@@ -671,32 +671,6 @@ EOF;
 		safe_update("txp_prefs","position=$i","name='custom_${i}_set'");
 	}
 
-
-	// RC4: adding privs table
-
-	if (!safe_query("DESCRIBE `".PFX."txp_priv`")) {
-		safe_query("CREATE TABLE `".PFX."txp_priv` (
-			`id` INT( 8 ) NOT NULL AUTO_INCREMENT ,
-			`priv` VARCHAR( 128 ) NOT NULL ,
-			`1` TINYINT( 1 ) NOT NULL ,
-			`2` TINYINT( 1 ) NOT NULL ,
-			`3` TINYINT( 1 ) NOT NULL ,
-			`4` TINYINT( 1 ) NOT NULL ,
-			`5` TINYINT( 1 ) NOT NULL ,
-			`6` TINYINT( 1 ) NOT NULL ,
-			PRIMARY KEY ( `id` )
-		) $tabletype");
-
-		include txpath.'/lib/admin_config.php';
-		
-		foreach($txp_permissions as $a => $b) {
-			$privs = explode(',',$b);
-			foreach ($privs as $c) $sets[] = "`$c` = 1";
-			safe_insert("txp_priv","priv='$a', ".join(', ',$sets));
-			unset($sets);
-		}
-	}
-
 	// index ip column in txp_log
 
 		$ipidxset = false;
@@ -712,7 +686,7 @@ EOF;
 	// Show gmt-selection in prefs
 	safe_update('txp_prefs',"type=0, html='gmtoffset_select', position=50","name='gmtoffset'");
 
-	if (!safe_field('name', 'txp_prefs', "prefs_id=1, name='plugin_cache_dir'")) {
+	if (!safe_field('name', 'txp_prefs', "prefs_id=1 and name='plugin_cache_dir'")) {
 		$maxpos = safe_field('max(position)', 'txp_prefs', '1');
 		safe_insert('txp_prefs',"name='plugin_cache_dir', val='', prefs_id='1', type='1', event='admin', position='".doSlash($maxpos)."', html='text_input'");
 	}
